unit hiMemoryRW;

{$IFNDEF FPC}{$HINTS OFF}{$ENDIF}
{$WARNINGS OFF}

interface

uses Windows, Share, Kol;

type
  
  ThiMemoryRW = class
    private
      FMemInfo: TMemoryBasicInformation;
    public
      _prop_Address: Integer;
      _prop_Data: TData;
      _prop_DataType: Integer;
      _prop_StringLen: Integer;


      _data_Address: THI_Event;
      _data_Data: THI_Event;
      _data_ProcID: THI_Event;
      _data_StringLen: THI_Event;
  
      _event_onError: THI_Event;
      _event_onResult: THI_Event;
      
      procedure _work_doDataType(var _Data: TData; Index: Word);
      procedure _work_doGetRegionInfo(var _Data: TData; Index: Word);
      procedure _work_doRead(var _Data: TData; Index: Word);
      procedure _work_doSetProtection(var _Data: TData; Index: Word);
      procedure _work_doWrite(var _Data: TData; Index: Word);

      procedure _var_BaseAddress(var _Data: TData; Index: Word);
      procedure _var_RegionSize(var _Data: TData; Index: Word);
      procedure _var_RegionState(var _Data: TData; Index: Word);
      procedure _var_RegionProtect(var _Data: TData; Index: Word);
      procedure _var_RegionType(var _Data: TData; Index: Word);
      procedure _var_AllocBase(var _Data: TData; Index: Word);
      procedure _var_AllocProtect(var _Data: TData; Index: Word);
  end;
  
  function VirtualQueryEx(hProcess: THandle; lpAddress: Pointer; 
                         var lpBuffer: TMemoryBasicInformation; dwLength: DWORD): DWORD; stdcall; 
                         external 'kernel32.dll' name 'VirtualQueryEx';

  {$ifdef F_P}
  function VirtualProtectEx(hProcess: THandle; lpAddress: Pointer;
  dwSize, flNewProtect: DWORD; lpflOldProtect: Pointer): BOOL; stdcall; external 'kernel32.dll' name 'VirtualProtectEx';
  {$endif}

implementation

const
  DATA_TYPE_BYTE = 0;
  DATA_TYPE_WORD = 1;
  DATA_TYPE_DWORD = 2;
  DATA_TYPE_SINGLE = 3;
  DATA_TYPE_DOUBLE = 4;
  DATA_TYPE_STRING = 5;


procedure ThiMemoryRW._work_doDataType;
begin
  _prop_DataType := ToInteger(_Data);
end;


procedure ThiMemoryRW._work_doGetRegionInfo;
var 
  pid: integer;
  HProc: THandle;
  Addr: DWORD;
begin
  pid := ReadInteger(_Data, _data_ProcID, 0);
  Addr := ReadInteger(_Data, _data_Address, _prop_Address);
  
  FillChar(FMemInfo, SizeOf(TMemoryBasicInformation), 0);
    
  HProc := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, False, pid);
  if HProc <> 0 then
  begin    
    if VirtualQueryEx(HProc, Pointer(Addr), FMemInfo, SizeOf(TMemoryBasicInformation)) <> 0
    then 
      _hi_CreateEvent(_Data, @_event_onResult)
    else
      _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError));
      
    CloseHandle(HProc);
  end
  else
    _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError)); 
end;


procedure ThiMemoryRW._work_doRead;
var 
  pid: integer;
  HProc: THandle;
  Addr, L, Count: DWORD;
  Buf: Pointer;
  RsltI: Integer;
  RsltSng: Single; 
  RsltDbl: Double; 
  RsltStr: string;
begin
  pid := ReadInteger(_Data, _data_ProcID, 0);
  Addr := ReadInteger(_Data, _data_Address, _prop_Address);
  
  RsltI := 0;
  RsltSng := 0;
  RsltDbl := 0;
  
  L := 0;
  case _prop_DataType of
    DATA_TYPE_BYTE:
      begin
        L := 1;
        Buf := @RsltI;
      end;
    DATA_TYPE_WORD:
      begin
        L := 2;
        Buf := @RsltI;
      end;
    DATA_TYPE_DWORD:
      begin
        L := 4;
        Buf := @RsltI;
      end;
    DATA_TYPE_SINGLE:
      begin
        L := 4;
        Buf := @RsltSng;
      end;
    DATA_TYPE_DOUBLE:
      begin
        L := 8;
        Buf := @RsltDbl;
      end;
    DATA_TYPE_STRING:
      begin
        L := ReadInteger(_Data, _data_StringLen, _prop_StringLen);
        SetLength(RsltStr, L);
        Buf := PChar(RsltStr);
      end;
  end;
  
  if L = 0 then Exit;
  
  HProc := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, False, pid);
  if HProc <> 0 then
  begin    
    if ReadProcessMemory(HProc, Pointer(Addr), Buf, L, Count) then
    begin
      case _prop_DataType of
        DATA_TYPE_BYTE..DATA_TYPE_DWORD: _hi_CreateEvent(_Data, @_event_onResult, RsltI);
        DATA_TYPE_SINGLE: _hi_CreateEvent(_Data, @_event_onResult, RsltSng);
        DATA_TYPE_DOUBLE: _hi_CreateEvent(_Data, @_event_onResult, RsltDbl);
        DATA_TYPE_STRING: _hi_CreateEvent(_Data, @_event_onResult, RsltStr);
      end;
    end
    else
      _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError));

    CloseHandle(HProc);
  end
  else _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError)); 
end;


procedure ThiMemoryRW._work_doSetProtection;
var 
  pid, Prot: Integer;
  HProc: THandle;
  Addr: DWORD;
  Info: TMemoryBasicInformation;
begin
  Prot := ToInteger(_Data);
  pid := ReadInteger(_Data, _data_ProcID, 0);
  Addr := ReadInteger(_Data, _data_Address, _prop_Address);
  if (Prot = 0) or (pid = 0) then Exit;
  
  FillChar(Info, SizeOf(TMemoryBasicInformation), 0);
    
  HProc := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_OPERATION, False, pid);
  if HProc <> 0 then
  begin    
    if VirtualQueryEx(HProc, Pointer(Addr), Info, SizeOf(TMemoryBasicInformation)) <> 0
    then
    begin
      if VirtualProtectEx(HProc, Info.BaseAddress, Info.RegionSize, Prot, @pid) then
        _hi_CreateEvent(_Data, @_event_onResult, pid)
      else
        _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError));
    end
    else
      _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError));
      
    CloseHandle(HProc);
  end
  else
    _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError)); 
end;


procedure ThiMemoryRW._work_doWrite;
var 
  pid: Integer;
  HProc: THandle;
  Addr, L, Count: DWORD;
  Buf: Pointer;
  TmpI: Integer;
  TmpSng: Single; 
  TmpDbl: Double; 
  TmpStr: string;
begin
  pid := ReadInteger(_Data, _data_ProcID, 0);
  Addr := ReadInteger(_Data, _data_Address, _prop_Address);

  L := 0;
  case _prop_DataType of
    DATA_TYPE_BYTE..DATA_TYPE_DWORD:
      begin
        TmpI := ReadInteger(_Data, _data_Data, ToInteger(_prop_Data));
        case _prop_DataType of
          DATA_TYPE_BYTE: L := 1;
          DATA_TYPE_WORD: L := 2;
          DATA_TYPE_DWORD: L := 4;
        end;
        Buf := @TmpI;
      end;
    DATA_TYPE_SINGLE:
      begin
        L := 4;
        TmpSng := ReadReal(_Data, _data_Data, ToReal(_prop_Data));
        Buf := @TmpSng;
      end;
    DATA_TYPE_DOUBLE:
      begin
        L := 8;
        TmpDbl := ReadReal(_Data, _data_Data, ToReal(_prop_Data));
        Buf := @TmpDbl;
      end;
    DATA_TYPE_STRING:
      begin
        TmpStr := ReadString(_Data, _data_Data, ToString(_prop_Data));
        L := Length(TmpStr);
        Buf := PChar(TmpStr);
      end;
  end;
  
  if L = 0 then Exit;
  
  HProc := OpenProcess(PROCESS_VM_WRITE or PROCESS_VM_OPERATION, False, pid);
  if HProc <> 0 then
  begin    
    if WriteProcessMemory(HProc, Pointer(Addr), Buf, L, Count) then
      _hi_CreateEvent(_Data, @_event_onResult)
    else
      _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError));

    CloseHandle(HProc);
  end
  else _hi_CreateEvent(_Data, @_event_onError, Integer(GetLastError)); 
end;

procedure ThiMemoryRW._var_BaseAddress;
begin
  dtInteger(_Data, Integer(FMemInfo.BaseAddress));
end;

procedure ThiMemoryRW._var_RegionSize;
begin
  dtInteger(_Data, FMemInfo.RegionSize);
end;

procedure ThiMemoryRW._var_RegionState;
begin
  dtInteger(_Data, FMemInfo.State);
end;

procedure ThiMemoryRW._var_RegionProtect;
begin
  dtInteger(_Data, FMemInfo.Protect);
end;

procedure ThiMemoryRW._var_RegionType;
begin
  dtInteger(_Data, FMemInfo.Type_9);
end;

procedure ThiMemoryRW._var_AllocBase;
begin
  dtInteger(_Data, Integer(FMemInfo.AllocationBase));
end;

procedure ThiMemoryRW._var_AllocProtect;
begin
  dtInteger(_Data, FMemInfo.AllocationProtect);
end;

{$IFNDEF FPC}{$HINTS ON}{$ENDIF}
{$WARNINGS ON}

end.